require 'spec_helper'

describe Gamifier::DSL::Network do

  let(:network) { Gamifier::DSL::Network.new }
  let(:url) { 'http://foo.com' }
  let(:key) { '1234' }
  let(:enterprise_key) { '5678' }

  before do
    Gamifier.set :uri => url, :key => key, :enterprise_key => enterprise_key
  end

  describe "#initialize" do

    subject { network }

    its(:source) { should_not be_nil }
    its(:sites) { should be_empty }
    its(:api_keys) { should be_empty }

  end

  describe "#engine" do

    subject { network.engine }

    it { should_not eq(Gamifier.engine) }
    it { should_not eq(network.enterprise_engine) }

  end

  describe "#enterprise_engine" do

    subject { network.enterprise_engine }

    it { should_not eq(Gamifier.engine) }
    it { should_not eq(network.engine) }

  end

  describe "#method_missing" do

    let(:args) { 'bar' }
    let(:block) { lambda {|x| x} }

    it "should call on source" do
      network.source.should_receive(:foo).with(args, &block)
      network.foo(args, &block)
    end

  end

  describe "#enterprise_engine" do

    subject { network.enterprise_engine }

    it { should_not eq(Gamifier.engine) }
    it { subject.config[:enterprise_key].should eq(enterprise_key) }

  end

  describe "#get_key" do

    let(:public_key) { Gamifier::ApiKey.new(:access => 'public', :key => 'public_0000') }
    let(:private_key) { Gamifier::ApiKey.new(:access => 'private', :key => 'private_0000') }

    subject { network.get_key }

    context "without keys" do

      it { should be_nil }

    end

    context "with a public key" do

      before do
        network.stub(:api_keys).and_return([public_key])
      end

      it { should be_nil }

    end

    context "with a private key" do

      before do
        network.stub(:api_keys).and_return([private_key])
      end

      it { should eq('private_0000') }

    end

    context "with both public and private key" do

      before do
        network.stub(:api_keys).and_return([public_key, private_key])
      end

      it { should eq('private_0000') }

    end

  end

  describe "#site" do

    it { expect { network.site('foo') }.to change{ network.sites.count }.by(1) }
    it { subject.site('foo').name.should eq('foo') }

    it "should call eval_with_context" do
      block = proc {}
      Gamifier::DSL.should_receive(:eval_with_context).with(instance_of(Gamifier::DSL::Site), &block)
      network.site('foo', &block)
    end

    it "should set values" do
      o = network.site('foo') do
        set :key, 'value'
      end
      o.key.should eq('value')
    end

  end

  describe "#api_key" do

    it { expect { network.api_key }.to change{ network.api_keys.count }.by(1) }

    it "should call eval_with_context" do
      block = proc {}
      Gamifier::DSL.should_receive(:eval_with_context).with(instance_of(Gamifier::DSL::ApiKey), &block)
      network.api_key(&block)
    end

    it "should set values" do
      o = network.api_key do
        set :key, 'value'
      end
      o.key.should eq('value')
    end

  end

end
