require 'spec_helper'

describe Gamifier::Engine do

  def site
    {
      "name" => "ibadgedyou",
      "created_at" => "2012-05-14T01:50:49-07:00",
      "url" => "ibadgedyou.dev",
      "id" => "4fb0c76949f83818c20000eb",
      "_id" => "4fb0c76949f83818c20000eb"
    }
  end

  before do
    @logger = Logger.new(STDERR)
    Gamifier.stub!(:config) {
      {:uri => "http://somewhere.ltd/path/", :key => "1234"}
    }
    Gamifier.stub!(:logger).and_return @logger

    @engine = Gamifier::Engine.new
  end

  describe "#uri_to" do

    it "should correctly concatenate URIs" do
      @engine.uri_to.to_s.should == "http://somewhere.ltd/path/1234/"
      @engine.uri_to("resource").to_s.should == "http://somewhere.ltd/path/1234/resource"
      @engine.uri_to("/resource").to_s.should == "http://somewhere.ltd/resource"
      @engine.uri_to("../resource").to_s.should == "http://somewhere.ltd/path/resource"
    end

  end

  describe "configuration" do

    before do
      @engine = Gamifier::Engine.new(:key => "new-key")
    end

    it "should overwrite the global configuration with the given parameters at intialization time" do
      @engine.config.should == {:uri => "http://somewhere.ltd/path/", :key => "new-key"}
      Gamifier.config.should == {:uri => "http://somewhere.ltd/path/", :key => "1234"}
    end

    describe "#ok?" do

      subject { @engine.ok? }

      context "with enterprise_key" do

        before do
          @engine.config[:key] = ''
          @engine.config[:enterprise_key] = '5678'
        end

        context "with uri" do

          before do
            @engine.config[:uri] = 'http://foo.com'
          end

          it { should be_true }

        end

        context "without uri" do

          before do
            @engine.config[:uri] = ''
          end

          it { should_not be_true }

        end

      end

      context "without enterprise_key" do

        before do
          @engine.config[:enterprise_key] = ''
        end

        context "with key and uri" do

          before do
            @engine.config[:key] = '1234'
            @engine.config[:uri] = 'http://foo.com'
          end

          it { should be_true }

        end

        context "with key" do

          before do
            @engine.config[:key] = '1234'
            @engine.config[:uri] = ''
          end

          it { should_not be_true }

        end

        context "with uri" do

          before do
            @engine.config[:key] = ''
            @engine.config[:uri] = 'http://foo.com'
          end

          it { should_not be_true }

        end

        context "without key and uri" do

          before do
            @engine.config[:key] = ''
            @engine.config[:uri] = ''
          end

          it { should_not be_true }

        end

      end

    end

  end

  it "should have initialized a connection" do
    @engine.connection.should be_a(Object)
  end

  it "should output to the given logger" do
    @logger.should_receive(:info).at_least(1).times

    stub_request(:get, "http://somewhere.ltd/path/1234/resource.json").
      with(:query => {"limit" => "5"})
    @engine.transmit :get, "resource", :query => {"limit" => "5"}
  end

  describe "#transmit" do

    it "transmits POST requests to the API" do
      stub_request(:post, "http://somewhere.ltd/path/1234/resource.json").
        with(:body => {:a => "b", :c => ["d", "e"]}, :headers => { 'Content-Type' => 'application/x-www-form-urlencoded' })
      @engine.transmit :post, "resource", :body => {"a" => "b", :c => ["d", "e"]}
    end

    it "transmits GET requests to the API" do
      stub_request(:get, "http://somewhere.ltd/path/1234/resource.json").
        with(:query => {"limit" => "5"})
      @engine.transmit :get, "resource", :query => {"limit" => "5"}
    end

    it "should return true if the response is 204" do
      stub_request(:get, "http://somewhere.ltd/resource.json").
        to_return(:status => 204)
      @engine.transmit(:get, "/resource").should be_true
    end

    it "should return the parsed response if the status is in the 2xx range (except 204)" do
      stub_request(:get, "http://somewhere.ltd/resource.json").
        to_return(:body => MultiJson.dump(site), :headers => {'Content-Type' => 'application/json;charset=utf-8'})
      @engine.transmit(:get, "/resource").should == site
    end

    [:get, :head].each do |method|
      it "should return nil if the response status is 422 (Unprocessable Entity) and method is #{method}" do
        stub_request(method, "http://somewhere.ltd/resource.json").
          to_return(:status => 422)
        @engine.transmit(method, "/resource").should be_nil
      end
    end

    [:delete, :post, :put].each do |method|
      it "should return false if the response status is 422 (Unprocessable Entity) and method is #{method}" do
        stub_request(method, "http://somewhere.ltd/resource.json").
          to_return(:status => 422)
        @engine.transmit(method, "/resource").should be_false
      end
    end

    it "should return nil if the response status is 404" do
      stub_request(:get, "http://somewhere.ltd/resource.json").
        to_return(:status => 404)
      @engine.transmit(:get, "/resource").should be_nil
    end

    it "should raise an HTTPServerError for a 5xx code" do
      stub_request(:get, "http://somewhere.ltd/resource.json").
        to_return(:status => 500)
      expect {
        @engine.transmit(:get, "/resource")
      }.to raise_error Gamifier::HTTPServerError
    end

    it "should raise an HTTPClientError for a 4xx code" do
      stub_request(:get, "http://somewhere.ltd/resource.json").
        to_return(:status => 400)
      expect {
        @engine.transmit(:get, "/resource")
      }.to raise_error Gamifier::HTTPClientError
    end

  end

  describe "collections" do

    %w{activities activity_definitions groups players rewards reward_definitions sites units users}.each do |collection|
      it "should respond ##{collection}" do
        @engine.should respond_to(collection.to_sym)
      end
    end

    it "should instantiate a new collection object with the correct model" do
      collection = @engine.players
      collection.should be_a(Gamifier::Collection)
      collection.engine.should == @engine
      collection.model.should == Gamifier::Player
    end

  end

end
