require 'spec_helper'

describe Gamifier::Model do

  def raw_find
    '{"data":{"name":null,"created_at":"2012-05-31T04:59:22-07:00","email":"1234@dimelo.com","_id":"4fc75d1a3dc648153b006183"},"paging":null}'
  end

  def raw_create
    '{"name": null, "created_at": "2012-05-31T05:37:51-07:00", "email": "1234@test.com", "_id": "4fc7661f49f83857cd006044"}'
  end

  before do
    Gamifier.set :uri => "http://somewhere.ltd/path/", :key => "1234"
    Gamifier::Model.reset!
  end

  it "should return the underscored version of the class name" do
    class ActivityDefinition < Gamifier::Model; end
    ActivityDefinition.path.should == "activity_definitions"
    Gamifier::Model.path.should == "models"
  end

  it "should allow to set a custom path" do
    Gamifier::Model.path.should == "models"
    Gamifier::Model.path "another/path.json"
    Gamifier::Model.path.should == "another/path.json"
    class ActivityDefinition < Gamifier::Model
      path "somewhere"
    end
    ActivityDefinition.path.should == "somewhere"
  end

  describe "model instances" do

    before do
      @model = Gamifier::Model.new :x => "y", :a => "b"
    end

    it "should return the attributes Hash" do
      @model.attributes.should == {:x => "y", :a => "b"}
    end

    it "should forward #engine to Gamifier" do
      @model.engine.should == Gamifier.engine
    end

    it "should behave like an openstruct" do
      @model.xyz = "yeah"
      @model.xyz.should == "yeah"
    end

    it "should be instantiable with a hash" do
      model = Gamifier::Model.new :k => "v", "other" => ["1", 2]
      model.k.should == "v"
      model.other.should == ["1", 2]
    end

    it "should compute the path to itself" do
      @model._id = "1234"
      @model.path.should == "models/1234"
    end

    it "should be new if no id yet" do
      @model.should be_new
    end

    it "should not be new if an id already exists" do
      @model._id = "1234"
      @model.should_not be_new
    end

    describe "existing model" do

      before do
        @model.stub!(:new?).and_return false
        @model.stub!(:path).and_return("models/1234")
      end

      it "should save itself using PUT if not new" do
        @model.engine.should_receive(:transmit).with(:put, "models/1234", :body => {"model" => hash_including(:x => "y")})
        @model.save
      end

      it "should destroy itself using DELETE if not new" do
        @model.engine.should_receive(:transmit).with(:delete, "models/1234")
        @model.destroy
      end

      it "should update the model with the given attributes" do
        @model.engine.should_receive(:transmit).with(:put, "models/1234", :body => {"model" => {:key => "value"}})
        @model.update_attributes(:key => "value")
      end

    end

    describe "new model" do

      before do
        @model.stub!(:new?).and_return true
      end

      it "should save itself using POST if new" do
        @model.engine.should_receive(:transmit).with(:post, "models", :body => {"model" => hash_including(:x => "y")})
        @model.save
      end

      it "should return true if destroying and new" do
        @model.destroy.should be_true
      end

      it "should raise an error if trying to call update_attributes" do
        expect{ @model.update_attributes(:key => "value") }.to raise_error(Gamifier::Error)
      end

    end

    describe "#payload_for_submission" do

      it "should reject any attribute not in the list of the mutable_attributes (if any)" do
        @model.class.mutable_attributes :last_name
        @model.first_name = "hello"
        @model.last_name = "world"
        @model.payload_for_submission.should == {"model" => {:last_name => "world"}}
      end

    end

    describe "replace_if_successful" do

      it "should replace the attributes if res is a Hash and has a 'data' key" do
        @model.replace_if_successful(MultiJson.load(raw_create)).should == @model
        @model._id.should == "4fc7661f49f83857cd006044"
      end

      [true, false, nil].each do |value|
        it "should do nothing if given response is #{value.inspect}" do
          @model.replace_if_successful(value).should == value
        end
      end

    end

  end

  describe "FinderMethods" do

    before do
      @engine = mock(Gamifier::Engine)
      @collection = Gamifier::Collection.new(@engine, Gamifier::Model)
    end

    describe "all" do

      it "should issue a get request with no params" do
        @engine.should_receive(:transmit).with(:get, "models", :query => {:page => 1, :per_page => 50})
        @collection.all
      end

      it "should issue a get request with the given params" do
        @engine.should_receive(:transmit).with(:get, "models", :query => {:page => 1, :per_page => 50, :limit => 5})
        @collection.all :limit => 5
      end

      it "should return a list of entries" do
        @engine.should_receive(:transmit).and_return({'data' => [{'key' => 'value1'}, {'key' => 'value2'}]})
        entries = @collection.all
        entries.length.should == 2
        entries.all?{|e| e.kind_of?(Gamifier::Model)}.should be_true
      end

    end

    describe "find" do

      it "should return the first entry if data is returned" do
        @engine.should_receive(:transmit).with(:get, "models/1234@dimelo.com", :query => {}).
          and_return MultiJson.load(raw_find)
        user = @collection.find("1234@dimelo.com")
        user.should be_a(Gamifier::Model)
        user._id.should == "4fc75d1a3dc648153b006183"
      end

    end

    describe "find_by_name" do

      before do
        @raw_payload = '{"data":[{"name":"karma","label":"Karma","normalized_name":"unit_karma","type":"point","abbreviation":"kar","display_priority":0},{"name":"yoh","label":"Yoh","normalized_name":"unit_yoh","type":"point","abbreviation":"yoh","display_priority":0},{"name":"karma","label":"Karma","normalized_name":"unit_karma","type":"point","abbreviation":"kar","display_priority":0}],"paging":{}}'
      end

      it "should return the first entry if a matching entry can be found" do
        @engine.should_receive(:transmit).with(:get, "models", instance_of(Hash)).
          and_return MultiJson.load(@raw_payload)
        entry = @collection.find_by_name("karma")
        entry.should be_a(Gamifier::Model)
        entry.label.should == "Karma"
      end

      it "should return nil if no entry can be found" do
        @engine.should_receive(:transmit).with(:get, "models", instance_of(Hash)).
          and_return MultiJson.load('{"data": [], "paging": {}}')
        entry = @collection.find_by_name("karma")
        entry.should be_nil
      end

    end

  end

end
